Replace this with your own readme.

==

This template project was generated by

    resilient-circuits codegen -p {{package}} [-f {{functions.keys()|join(" ")}}] [-w {{workflows.keys()|join(" ")}}]


You must edit the `setup.py` before distribution;
it should contain your actual contact and license information.

To install in "development mode"

    pip install -e {{output_dir}}/

After installation, the package will be loaded by `resilient-circuits run`.


To uninstall,

    pip uninstall {{package}}


To package for distribution,

    python {{output_dir}}/setup.py sdist

The resulting .tar.gz file can be installed using

    pip install <filename>.tar.gz

